/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package picto.app.gui;

import java.awt.Color;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.ObjectInputStream;

import java.util.Calendar;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.filechooser.FileNameExtensionFilter;

import picto.utils.Picto;
import picto.app.interfaces.Locales;
import picto.app.interfaces.Localizable;
import picto.app.interfaces.Playable;
import picto.utils.ColoredButton;
import picto.utils.Config;
import picto.utils.FileDrop;
import picto.utils.PatternDataSetter;
import picto.utils.PhraseScrambler;
import picto.utils.Saver;
import picto.utils.SavingUtils;
import picto.utils.Text;

import static picto.utils.Text._;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class MainWindow extends JFrame implements Localizable {

    private static final long serialVersionUID = -765348566788957685L;
    private int xSize, ySize;
    private Picto pic;
    private JPanel board;

    /** Creates new form MainWindow */
    public MainWindow() {
        initComponents();
        generateMenues();
        setSize(600, 400);
        addWeakReference();
        updateColors();
        updateTexts();
        updateMenues();
        addActionListeners();
        initFileDrop();
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        setBackground(Config.getColor("backgroundColor"));
        setIconImage(new ImageIcon(
                GameLoader.class.getResource("/res/images/icon.png")).getImage());
    }

    @Override
    public final void updateTexts() {
        appMenu.setText(_("Game"));
        appMenu.setMnemonic('G');
        openRandomPhraseMenuItem.setText(_("Open random phrase"));
        openRandomPhraseMenuItem.setMnemonic('s');
        openRandomGameMenuItem.setText(_("Open random pattern"));
        openPatternMenuItem.setText(_("Open pattern"));
        printPatternMenuItem.setText(_("Print pattern"));
        loadGameMenuItem.setText(_("Open..."));
        loadGameMenuItem.setToolTipText(_("Open saved game"));
        saveGameMenuItem.setText(_("Save..."));
        saveGameMenuItem.setToolTipText(_("Save current game"));
        quitMenuItem.setText(_("Quit"));
        editMenu.setText(_("Edit"));
        editMenu.setMnemonic('d');
        localeMenu.setText(_("Locale"));
        enMenuItem.setText(_("English"));
        esMenuItem.setText(_("Spanish"));
        trMenuItem.setText(_("Turkish"));
        zhMenuItem.setText(_("Chinese (Simplified)"));
        jpMenuItem.setText(_("Japanese"));
//        glMenuItem.setText(_("Galician"));
        colorsMenuItem.setText(_("Colors"));
        dragMenuItem.setText(_("Enable dragging"));
        mouseTrackerMenuItem.setText(_("Enable mouse tracking"));
        patternMenu.setText(_("Pattern"));
        patternMenu.setMnemonic('t');
        clearMenuItem.setText(_("Clear board"));
        checkMenuItem.setText(_("Check board"));
        checkMenuItem.setToolTipText(_("Check if some cell is wrongly marked"));
        newPhraseMenuItem.setText(_("Custom phrase"));
        solve2MenuItem.setText(_("Attempt to solve"));
        solve2MenuItem.setToolTipText(_("Experimental"));
        solve2MenuItem.setForeground(Color.RED);
        newPatternMenuItem.setText(_("New pattern"));
        showPreviewMenuItem.setText(_("Show preview"));
        extrasMenu.setText(_("Extras..."));
        extrasMenu.setMnemonic('x');
        checkAndResetMenuItem.setText(_("Check board and reset wrong"));
        checkAndResetMenuItem.setToolTipText(_("Check if some cell is wrongly"
                + " marked, and resets it"));
        solveMenuItem.setText(_("Show solution"));
        helpMenu.setMnemonic('?');
        helpMenuItem.setText(_("Help"));
        aboutMenuItem.setText(_("About..."));
        setTitle();
        switch (Config.getLocale()) {
            case Locales.ENGLISH:
                enMenuItem.doClick();
                break;
            case Locales.SPANISH:
                esMenuItem.doClick();
                break;
            case Locales.TURKISH:
                trMenuItem.doClick();
                break;
            case Locales.SIMPLIFIED_CHINESE:
                zhMenuItem.doClick();
                break;
            case Locales.JAPANESE:
                jpMenuItem.doClick();
                break;
            default:
                assert false : "Unrecognized locale!";
        }
        openRandomPhraseMenuItem.setEnabled(!PhraseScrambler.isEmpty());
    }

    @Override
    public final void addWeakReference() {
        Text.addWeakReference(this);
    }

    public void setGame(Picto pic) {
        if (this.pic != null) {
            this.pic.flush();
        }
        this.pic = pic;
        this.pic.regenerate();
        if (board != null) {
            backgroundPanel.remove(board);
        }
        board = new PlayingBoard(pic);
        addPanel();
    }

    private void addPanel() {
        xSize = ((Playable) board).getXSize();
        ySize = ((Playable) board).getYSize();
        board.setVisible(true);
        backgroundPanel.add(board);
        centerPanel();
        backgroundPanel.updateUI();
    }
    private static String ERROR_MSG2 = _("File doesn't appear to be\n"
            + "a valid PictoSaveFile");

    /**
     * Saves the current pattern
     */
    private void saveGame() {
        JFileChooser fc = new JFileChooser();
        fc.setDialogTitle(_("Select file to save"));
        fc.setFileHidingEnabled(true);
        fc.setMultiSelectionEnabled(false);

        // Saves an instance of a current game
        if (board instanceof PlayingBoard) {
            fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
            fc.setFileFilter(new FileNameExtensionFilter(
                    _("PictoGame save file"), "PictoSave"));
            fc.setSelectedFile(new File(
                    Calendar.getInstance().getTime().getTime() + ".PictoSave"));
            if (fc.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
                int response = JOptionPane.YES_OPTION;
                if (fc.getSelectedFile().exists()) {
                    response = JOptionPane.showConfirmDialog(null,
                            _("Overwrite existing file?"), _("Confirm overwrite"),
                            JOptionPane.OK_CANCEL_OPTION,
                            JOptionPane.QUESTION_MESSAGE);
                }
                if (response == JOptionPane.YES_OPTION) {
                    SavingUtils.saveGame(fc.getSelectedFile(), ((PlayingBoard) board).getSaver());
                }
            }
        } else // Saves a pattern
        if (board instanceof CreatePatternBoard) {
            fc.setFileFilter(new FileNameExtensionFilter(
                    _("PictoGame pattern file"), "pattern"));
            fc.setSelectedFile(new File(_("my_pattern") + ".pattern"));
            if (fc.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
                int response = JOptionPane.YES_OPTION;
                if (fc.getSelectedFile().exists()) {
                    response = JOptionPane.showConfirmDialog(null,
                            _("Overwrite existing file?"), _("Confirm overwrite"),
                            JOptionPane.OK_CANCEL_OPTION,
                            JOptionPane.QUESTION_MESSAGE);
                }
                if (response == JOptionPane.YES_OPTION) {
                    SavingUtils.saveNewPattern(fc.getSelectedFile(), ((CreatePatternBoard) board).getPic());
                }
            }
        }
    }

    private void loadGame() {
        JFileChooser fc = new JFileChooser();
        fc.setDialogTitle(_("Select file to load"));
        fc.setFileHidingEnabled(true);
        fc.setMultiSelectionEnabled(false);
        fc.setFileFilter(new FileNameExtensionFilter(_("PictoGame save file"), "PictoSave", "pattern"));
        if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
            File f = fc.getSelectedFile();
            try {
                if (f.getCanonicalPath().endsWith(".PictoSave")) {
                    ObjectInputStream ois = new ObjectInputStream(new FileInputStream(f));
                    if (board != null) {
                        backgroundPanel.remove(board);
                    }
                    Saver temp = (Saver) ois.readObject();
                    ois.close();
                    board = new PlayingBoard(temp);
                    if (pic != null) {
                        pic.flush();
                        pic = null;
                    }
                    if (temp.getPs() == null) {
                        pic = temp.getPic();
                    }
                    xSize = ((Playable) board).getXSize();
                    ySize = ((Playable) board).getYSize();
                    board.setVisible(true);
                    backgroundPanel.add(board);
                    centerPanel();
                    backgroundPanel.updateUI();
                } else if (f.getCanonicalPath().endsWith(".pattern")) {
                    Picto foo = new Picto(new BufferedReader(new FileReader(f)));
                    if (foo != null) {
                        if (board != null) {
                            backgroundPanel.remove(board);
                        }
                        if (pic != null) {
                            pic.flush();
                        }
                        pic = foo;
                        board = new CreatePatternBoard(foo);
                        xSize = ((Playable) board).getXSize();
                        ySize = ((Playable) board).getYSize();
                        setTitle();
                        board.setVisible(true);
                        backgroundPanel.add(board);
                        centerPanel();
                        repaint();
                    }
                    updateMenues();
                }
            } catch (IOException ex) {
                Logger.getLogger(MainWindow.class.getName()).log(Level.SEVERE, null, ex);
            } catch (Exception ex) {
                JOptionPane.showMessageDialog(null, _(ERROR_MSG2), _("Error"), JOptionPane.ERROR_MESSAGE);
                Logger.getLogger(MainWindow.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    /**
     * Updates all the colors after generateList change in the Colors window
     * @see picto.app.gui.ColorChooser
     */
    public final void updateColors() {
        if (board != null) {
            ((Playable) board).updateColors();
        }
        backgroundPanel.setBackground(Config.getColor("backgroundColor"));
    }

    private static final String TITLE = "PictoGame v" + Config.getAppVersion();
    /**
     * Sets the title according to the situation
     */
    private void setTitle() {
        setTitle(pic == null ? TITLE : TITLE + " [" + pic.getTitle() + "]");
        updateMenues();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        backgroundPanel = new KeyPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        javax.swing.GroupLayout backgroundPanelLayout = new javax.swing.GroupLayout(backgroundPanel);
        backgroundPanel.setLayout(backgroundPanelLayout);
        backgroundPanelLayout.setHorizontalGroup(
            backgroundPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 713, Short.MAX_VALUE)
        );
        backgroundPanelLayout.setVerticalGroup(
            backgroundPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 430, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(backgroundPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(backgroundPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel backgroundPanel;
    // End of variables declaration//GEN-END:variables

    /**
     * Centers the game panel to ensure that the cells are always square
     */
    private void centerPanel() {
        if ((xSize | ySize) == 0) {
            return;
        }
        int w = backgroundPanel.getWidth(), h = backgroundPanel.getHeight();
        int newS = (w / xSize * ySize > h) ? h / ySize : w / xSize;
        board.setLocation((w - (newS * xSize)) / 2, (h - (newS * ySize)) / 2);
        board.setSize(newS * xSize, newS * ySize);
        board.validate();
    }

    /**
     * Generates the main menu
     */
    private void generateMenues() {
        menuBar.add(appMenu);
        appMenu.add(openRandomPhraseMenuItem);
        appMenu.add(openRandomGameMenuItem);
        appMenu.add(openPatternMenuItem);
        appMenu.add(printPatternMenuItem);
        appMenu.addSeparator();
        appMenu.add(loadGameMenuItem);
        appMenu.add(saveGameMenuItem);
        appMenu.addSeparator();
        appMenu.add(quitMenuItem);
        menuBar.add(editMenu);
        editMenu.add(localeMenu);
        localeMenu.add(enMenuItem);
        localeMenu.add(esMenuItem);
        localeMenu.add(trMenuItem);
        localeMenu.add(zhMenuItem);
        localeMenu.add(jpMenuItem);
//        localeMenu.add(glMenuItem);
        localeButtonGroup.add(enMenuItem);
        localeButtonGroup.add(esMenuItem);
        localeButtonGroup.add(trMenuItem);
        localeButtonGroup.add(zhMenuItem);
        localeButtonGroup.add(jpMenuItem);
//        localeButtonGroup.add(glMenuItem);
        editMenu.add(dragMenuItem);
        editMenu.add(mouseTrackerMenuItem);
        editMenu.add(colorsMenuItem);
        menuBar.add(patternMenu);
        patternMenu.add(clearMenuItem);
        patternMenu.add(checkMenuItem);
        patternMenu.add(newPhraseMenuItem);
        patternMenu.add(newPatternMenuItem);
        patternMenu.add(showPreviewMenuItem);
        patternMenu.add(solve2MenuItem);
        menuBar.add(helpMenu);
        helpMenu.add(helpMenuItem);
        helpMenu.add(aboutMenuItem);
        setJMenuBar(menuBar);
    }

    /**
     * Adds all the action listeners
     */
    private void addActionListeners() {
        openRandomPhraseMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (board != null) {
                    backgroundPanel.remove(board);
                }
                if (pic != null) {
                    pic.flush();
                    pic = null;
                }
                board = new PlayingBoard(PhraseScrambler.getRandomScrambler());
                addPanel();
                setTitle();
                updateMenues();
            }
        });
        openRandomGameMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                gameLoader.getRandomGame();
                setTitle();
                updateMenues();
            }
        });
        openPatternMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                gameLoader.setVisible(true);
                setTitle();
                updateMenues();
            }
        });
        printPatternMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                ((PlayingBoard) board).printPanel();
            }
        });
        newPhraseMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                String foo = JOptionPane.showInputDialog(MainWindow.this, _("Enter phrase"), _("Custom phrase"), JOptionPane.OK_CANCEL_OPTION);
                if (foo != null) {
                    if (board != null) {
                        backgroundPanel.remove(board);
                    }
                    if (pic != null) {
                        pic.flush();
                        pic = null;
                    }
                    board = new PlayingBoard(new PhraseScrambler(foo));
                    addPanel();
                    setTitle();
                    updateMenues();
                }
            }
        });
        newPatternMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                Picto foo = PatternDataSetter.getInstance().showMessage();
                if (foo != null) {
                    if (board != null) {
                        backgroundPanel.remove(board);
                    }
                    if (pic != null) {
                        pic.flush();
                        pic = null;
                    }
                    pic = foo;
                    board = new CreatePatternBoard(foo);
                    xSize = ((Playable) board).getXSize();
                    ySize = ((Playable) board).getYSize();
                    setTitle();
                    board.setVisible(true);
                    backgroundPanel.add(board);
                    centerPanel();
                    repaint();
                }
                updateMenues();
            }
        });
        loadGameMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                loadGame();
                updateMenues();
            }
        });
        saveGameMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                saveGame();
            }
        });
        quitMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                quit();
            }
        });
        esMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                Config.setLocale(Locales.SPANISH);
            }
        });
        enMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                Config.setLocale(Locales.ENGLISH);
            }
        });
        trMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                Config.setLocale(Locales.TURKISH);
            }
        });
        zhMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                Config.setLocale(Locales.SIMPLIFIED_CHINESE);
            }
        });
        jpMenuItem.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                Config.setLocale(Locales.JAPANESE);
            }
        });
        clearMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (JOptionPane.showConfirmDialog(backgroundPanel, _("Are you sure you want\nto clear the board?"),
                        _("Confirm"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                    ((Playable) board).clean();
                }
            }
        });
        solve2MenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (((Playable) board).getPic().getNumberOfRows() >= 64
                        || ((Playable) board).getPic().getNumberOfColumns() >= 64) {
                    JOptionPane.showMessageDialog(backgroundPanel,
                            _("Sorry but currently the maximum size supported is 64x64"), _("Sorry..."), JOptionPane.INFORMATION_MESSAGE);
                } else if (JOptionPane.showConfirmDialog(backgroundPanel, _("EXPERIMENTAL_FEATURE"),
                        _("Confirm"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                    attemptSolving();
                }
            }
        });
        checkMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                ((Playable) board).checkBoard(false);
            }
        });
        showPreviewMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                PreviewWindow.getInstance().setVisible(true);
            }
        });
        checkAndResetMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                ((Playable) board).checkBoard(true);
            }
        });
        solveMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                ((Playable) board).solve();
            }
        });
        helpMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (helpWindow == null) {
                    helpWindow = new HelpWindow();
                }
                helpWindow.setVisible(true);
            }
        });
        aboutMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (aboutWindow == null) {
                    aboutWindow = new AboutWindow();
                }
                aboutWindow.setVisible(true);
            }
        });
        colorsMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (colorChooserWindow == null) {
                    colorChooserWindow = new ColorChooser(MainWindow.this, false);
                }
                colorChooserWindow.setVisible(true);
            }
        });
        dragMenuItem.setSelected(Config.isDraggingEnabled());
        dragMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                Config.setDraggingEnabled(dragMenuItem.isSelected());
                ColoredButton.setDragEnabled(dragMenuItem.isSelected());
            }
        });
        mouseTrackerMenuItem.setSelected(Config.isMouseTrackingEnabled());
        mouseTrackerMenuItem.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                try {
                    Config.setMouseTrackingEnabled(mouseTrackerMenuItem.isSelected());
                    if (board != null) {
                        ((PlayingBoard) board).setTrakingEnabled(mouseTrackerMenuItem.isSelected());
                    }
                } catch (ClassCastException ex) {
                }
            }
        });
        addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(WindowEvent e) {
                quit();
            }
        });
        addComponentListener(new ComponentAdapter() {

            @Override
            public void componentResized(ComponentEvent evt) {
                centerPanel();
            }
        });
    }

    private void attemptSolving() {
        ((PlayingBoard) board).attemptSolving() ;
    }
    private int $ = 0;

    private void $() {
        JOptionPane.showMessageDialog(this, _("Cheater!"), _("AHA!"), JOptionPane.INFORMATION_MESSAGE);
        menuBar.add(extrasMenu, menuBar.getComponentIndex(helpMenu));
        extrasMenu.add(checkAndResetMenuItem);
        extrasMenu.add(solveMenuItem);
        menuBar.validate();
    }

    /**
     * Updates the state of the buttons
     */
    private void updateMenues() {
        boolean status = board != null;
        saveGameMenuItem.setEnabled(status);
        clearMenuItem.setEnabled(status);
        status &= board instanceof Playable;
        checkAndResetMenuItem.setEnabled(status);
        checkMenuItem.setEnabled(status);
        solveMenuItem.setEnabled(status);
        solve2MenuItem.setEnabled(board != null && board instanceof PlayingBoard);
//        printPatternMenuItem.setEnabled(board != null && board instanceof PlayingBoard);
        printPatternMenuItem.setEnabled(false);
    }

    private void initFileDrop() {
        FileDrop fd = new FileDrop(this, new FileDrop.Listener() {

            @Override
            public void filesDropped(File[] files) {
                try {
                    loadDroppedGame(files[0]);//Just use the firs element
                } catch (Exception ignoreMe) {}
            }
        });

    }

    private void loadDroppedGame(File f) {
        try {
            if (f.getCanonicalFile().toString().endsWith("pattern")) {
                Picto temp = new Picto(new BufferedReader(
                        new FileReader(f)));
                setGame(temp);
            } else if (f.getCanonicalFile().toString().endsWith("sgriddler")) {
                setGame(Picto.loadBoardFromSGriddlerFile(f));
            } else if (f.getCanonicalFile().toString().endsWith("PictoSave")) {
                ObjectInputStream ois = new ObjectInputStream(new FileInputStream(f));
                    if (board != null) {
                        backgroundPanel.remove(board);
                    }
                    Saver temp = (Saver) ois.readObject();
                    ois.close();
                    board = new PlayingBoard(temp);
                    if (pic != null) {
                        pic.flush();
                        pic = null;
                    }
                    if (temp.getPs() == null) {
                        pic = temp.getPic();
                    }
                    xSize = ((Playable) board).getXSize();
                    ySize = ((Playable) board).getYSize();
                    board.setVisible(true);
                    backgroundPanel.add(board);
                    centerPanel();
                    backgroundPanel.updateUI();
            } else {
                JOptionPane.showMessageDialog(null, _("That's not a valid PictoGame File"),
                        _("Oops..."), JOptionPane.ERROR_MESSAGE);
            }
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(null, _("Error opening file!"),
                    _("Oops..."), JOptionPane.ERROR_MESSAGE);
        }
    }

    /**
     * Quits the game
     */
    private void quit() {
        if (JOptionPane.showConfirmDialog(this,
                _("Are you sure you want to quit PictoGame?"),
                _("Confirm"), JOptionPane.YES_NO_OPTION,
                JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION) {
            Config.saveConfig();
            System.exit(0);
        }
    }
    private JMenuBar menuBar = new JMenuBar();
    private JMenu appMenu = new JMenu();
    private JMenuItem openRandomPhraseMenuItem = new JMenuItem();
    private JMenuItem openRandomGameMenuItem = new JMenuItem();
    private JMenuItem printPatternMenuItem = new JMenuItem();
    private JMenuItem openPatternMenuItem = new JMenuItem();
    private JMenuItem loadGameMenuItem = new JMenuItem();
    private JMenuItem saveGameMenuItem = new JMenuItem();
    private JMenuItem quitMenuItem = new JMenuItem();
    private JMenu editMenu = new JMenu();
    private JMenu localeMenu = new JMenu();
    private JCheckBoxMenuItem enMenuItem = new JCheckBoxMenuItem();
    private JCheckBoxMenuItem esMenuItem = new JCheckBoxMenuItem();
    private JCheckBoxMenuItem trMenuItem = new JCheckBoxMenuItem();
    private JCheckBoxMenuItem zhMenuItem = new JCheckBoxMenuItem();
    private JCheckBoxMenuItem jpMenuItem = new JCheckBoxMenuItem();
//    private JCheckBoxMenuItem glMenuItem = new JCheckBoxMenuItem();
    private ButtonGroup localeButtonGroup = new ButtonGroup();
    private JCheckBoxMenuItem dragMenuItem = new JCheckBoxMenuItem();
    private JCheckBoxMenuItem mouseTrackerMenuItem = new JCheckBoxMenuItem();
    private JMenuItem colorsMenuItem = new JMenuItem();
    private JMenu patternMenu = new JMenu();
    private JMenuItem clearMenuItem = new JMenuItem();
    private JMenuItem checkMenuItem = new JMenuItem();
    private JMenuItem newPatternMenuItem = new JMenuItem();
    private JMenuItem newPhraseMenuItem = new JMenuItem();
    private JMenuItem solveMenuItem = new JMenuItem();
    private JMenuItem showPreviewMenuItem = new JMenuItem();
    private JMenu extrasMenu = new JMenu();
    private JMenuItem checkAndResetMenuItem = new JMenuItem();
    private JMenuItem solve2MenuItem = new JMenuItem();
    private JMenu helpMenu = new JMenu("?");
    private JMenuItem helpMenuItem = new JMenuItem();
    private JMenuItem aboutMenuItem = new JMenuItem();
    private GameLoader gameLoader = new GameLoader(this);
    private AboutWindow aboutWindow;
    private HelpWindow helpWindow;
    private ColorChooser colorChooserWindow;

    private class KeyPanel extends JPanel {

        private static final long serialVersionUID = 748356312457871L;

        private KeyPanel() {
            setFocusable(true);
            addKeyListener(new KeyAdapter() {

                @Override
                public void keyPressed(KeyEvent evt) {
                    if ($ == -1 || board == null) {
                        return;
                    }
                    
                    switch (evt.getKeyCode()) {
                        case 0x25:$=($==4||$==6)?$+1:0;break;
                        case 0x26:$=($==0||$==1)?$+1:0;break;
                        case 0x27:$=($==5||$==7)?$+1:0;break;
                        case 0x28:$=($==2||$==3)?$+1:0;break;
                        case 0x42:$=($==8)?$+1:0;break;
                        case 0x41:$=($==9)?-1:0;break;
                        default:$=0;
                    }
                    
                    if($==-1)$();
                }
            });
        }
    }
}
